import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { catchError } from 'rxjs/operators';
import { SettingsDTO } from 'src/app/models/config/settings.dto';
import { ErrorDTO } from 'src/app/models/error/error-dto';
import { ServiceErrorCode } from 'src/app/models/error/service-error-code.enum';
import { AuthService } from 'src/app/services/auth/auth.service';
import { GuiService } from 'src/app/services/gui/gui.service';
import { SettingsService } from 'src/app/services/settings/settings.service';
import { SpinnerService } from 'src/app/services/spinner/spinner.service';
import { ToastService } from 'src/app/services/toast/toast.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.scss']
})
export class SettingsComponent implements OnInit {

  testMailForm!: FormGroup;
  data: SettingsDTO = {
    webVersion: '',
    apiVersion: '',
    configPath: '',
    webBaseURL: '',
    apiBaseURL: '',

    tokenExpireTime: 0,
    refreshTokenExpireTime: 0,

    mailUser: '',
    mailPort: 0,
    mailHost: '',
    mailTransceiver: '',
    mailTransceiverAddress: '',
  };

  constructor(
    private settingsService: SettingsService,
    private spinnerService: SpinnerService,
    private guiService: GuiService,
    private formBuilder: FormBuilder,
    private toastService: ToastService,
    private translate: TranslateService
  ) { }

  ngOnInit(): void {
    this.spinnerService.showSpinner();
    this.initForms();

    this.guiService.getSettings()
      .pipe(catchError(err => {
        this.spinnerService.hideSpinner();
        throw err;
      }))
      .subscribe(settings => {
        this.spinnerService.hideSpinner();
        this.data = settings;
        this.data.webVersion = this.settingsService.getWebVersion()?.getVersionString() ?? '0.0.0';
        this.data.apiBaseURL = this.settingsService.getApiURL();
        if (!this.data.apiBaseURL.endsWith('/')) {
          this.data.apiBaseURL += '/';
        }
      });
  }

  initForms(): void {
    this.testMailForm = this.formBuilder.group({
      mail: [null, [Validators.required, Validators.email]],
    });
  }

  testMail(): void {
    this.spinnerService.showSpinner();

    const mail = this.testMailForm.value.mail;
    if (!mail) {
      this.spinnerService.hideSpinner();
      return;
    }

    this.guiService.sendTestMail(mail)
      .pipe(catchError(error => {
        let header = this.translate.instant('admin.settings.message.error');
        let message = this.translate.instant('admin.settings.message.could_not_send_mail');

        if (error.error !== null) {
          const err: ErrorDTO = error.error;

          if (err.errorCode === ServiceErrorCode.ConnectionFailed) {
            header = this.translate.instant('admin.settings.message.connection_failed');
            message = this.translate.instant('admin.settings.message.connection_to_mail_failed');
            error.error = null;
          }

          if (err.errorCode === ServiceErrorCode.InvalidUser) {
            header = this.translate.instant('admin.settings.message.connection_failed');
            message = this.translate.instant('admin.settings.message.mail_login_failed');
            error.error = null;
          }

          if (err.errorCode === ServiceErrorCode.MailError) {
            header = this.translate.instant('admin.settings.message.send_failed');
            message = this.translate.instant('admin.settings.message.test_mail_not_send');
            error.error = null;
          }
        }

        this.spinnerService.hideSpinner();
        this.toastService.error(header, message);
        throw error;
      }))
      .subscribe(res => {
        this.spinnerService.hideSpinner();
        this.toastService.success(this.translate.instant('admin.settings.message.success'), this.translate.instant('admin.settings.message.send_mail'));
        this.testMailForm.reset();
      });
  }

}
